package app.fedilab.fedilabtube;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.annotation.SuppressLint;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.text.style.UnderlineSpan;
import android.util.Patterns;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;

import app.fedilab.fedilabtube.client.RetrofitPeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Error;
import app.fedilab.fedilabtube.client.entities.Oauth;
import app.fedilab.fedilabtube.client.entities.OauthParams;
import app.fedilab.fedilabtube.client.entities.Token;
import app.fedilab.fedilabtube.databinding.ActivityLoginBinding;
import app.fedilab.fedilabtube.helper.Helper;
import es.dmoral.toasty.Toasty;

import static app.fedilab.fedilabtube.client.RetrofitPeertubeAPI.updateCredential;


public class LoginActivity extends AppCompatActivity {


    private static String client_id;
    private static String client_secret;
    private ActivityLoginBinding binding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityLoginBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);

        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);


        SpannableString content_create = new SpannableString(getString(R.string.join_peertube));
        content_create.setSpan(new UnderlineSpan(), 0, content_create.length(), 0);
        content_create.setSpan(new ForegroundColorSpan(ContextCompat.getColor(LoginActivity.this, Helper.getColorAccent())), 0, content_create.length(),
                Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
        binding.createAnAccountPeertube.setText(content_create, TextView.BufferType.SPANNABLE);

        binding.createAnAccountPeertube.setOnClickListener(v -> {
            Intent mainActivity = new Intent(LoginActivity.this, PeertubeRegisterActivity.class);
            Bundle b = new Bundle();
            mainActivity.putExtras(b);
            startActivity(mainActivity);
        });


        if (BuildConfig.full_instances) {
            binding.loginInstanceContainer.setVisibility(View.VISIBLE);
        }


        if (Helper.isTablet(LoginActivity.this)) {

            ViewGroup.LayoutParams layoutParamsI = binding.loginInstanceContainer.getLayoutParams();
            layoutParamsI.width = (int) Helper.convertDpToPixel(300, LoginActivity.this);
            binding.loginInstanceContainer.setLayoutParams(layoutParamsI);

            ViewGroup.LayoutParams layoutParamsU = binding.loginUidContainer.getLayoutParams();
            layoutParamsU.width = (int) Helper.convertDpToPixel(300, LoginActivity.this);
            binding.loginUidContainer.setLayoutParams(layoutParamsU);

            ViewGroup.LayoutParams layoutParamsP = binding.loginPasswdContainer.getLayoutParams();
            layoutParamsP.width = (int) Helper.convertDpToPixel(300, LoginActivity.this);
            binding.loginPasswdContainer.setLayoutParams(layoutParamsP);
        }


        if (!BuildConfig.full_instances) {
            binding.loginUid.setOnFocusChangeListener((v, hasFocus) -> {
                if (!hasFocus) {
                    if (binding.loginUid.getText() != null && android.util.Patterns.EMAIL_ADDRESS.matcher(binding.loginUid.getText().toString().trim()).matches()) {
                        String[] emailArray = binding.loginUid.getText().toString().split("@");
                        if (emailArray.length > 1 && Arrays.asList(Helper.openid).contains(emailArray[1])) {
                            binding.loginButton.callOnClick();
                        }
                    }
                }
            });
        }


        binding.loginButton.setOnClickListener(v -> {

            if (binding.loginUid.getText() != null && binding.loginUid.getText().toString().contains("@") && !android.util.Patterns.EMAIL_ADDRESS.matcher(binding.loginUid.getText().toString().trim()).matches()) {
                Toasty.error(LoginActivity.this, getString(R.string.email_error)).show();
                return;
            }
            binding.loginButton.setEnabled(false);
            String instance, host;
            if (!BuildConfig.full_instances) {
                String[] emailArray = binding.loginUid.getText().toString().split("@");
                if (emailArray.length > 1 && !Arrays.asList(Helper.valideEmails).contains(emailArray[1])) {
                    Toasty.error(LoginActivity.this, getString(R.string.email_error_domain, emailArray[1])).show();
                    binding.loginButton.setEnabled(true);
                    return;
                }
                host = emailArray[1];
                instance = Helper.getPeertubeUrl(host);
            } else {
                if (binding.loginInstance.getText() == null || binding.loginInstance.getText().toString().trim().length() == 0) {
                    Toasty.error(LoginActivity.this, getString(R.string.not_valide_instance)).show();
                    binding.loginButton.setEnabled(true);
                    return;
                }
                instance = host = binding.loginInstance.getText().toString().trim().toLowerCase();
            }

            if (instance.startsWith("http")) {
                try {
                    URL url = new URL(instance);
                    instance = url.getHost();
                    host = instance;
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }
            } else if (instance.endsWith("/")) {
                try {
                    URL url = new URL("https://" + instance);
                    instance = url.getHost();
                    host = instance;
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }
            }
            if (!Patterns.WEB_URL.matcher("https://" + instance).matches()) {
                Toasty.error(LoginActivity.this, getString(R.string.not_valide_instance)).show();
                binding.loginButton.setEnabled(true);
                return;
            }
            String finalInstance = instance;
            String finalHost = host;
            if (Arrays.asList(Helper.openid).contains(host) && !BuildConfig.full_instances) {
                new Thread(() -> {
                    try {
                        Oauth oauth = new RetrofitPeertubeAPI(LoginActivity.this, finalInstance, null).oauthClient(null, null, null, null);
                        if (oauth == null) {
                            runOnUiThread(() -> {
                                binding.loginButton.setEnabled(true);
                                Toasty.error(LoginActivity.this, getString(R.string.client_error), Toast.LENGTH_LONG).show();
                            });
                            return;
                        }
                        client_id = oauth.getClient_id();
                        client_secret = oauth.getClient_secret();
                        SharedPreferences sharedpreferences = getSharedPreferences(Helper.APP_PREFS, MODE_PRIVATE);
                        SharedPreferences.Editor editor = sharedpreferences.edit();
                        editor.putString(Helper.CLIENT_ID, client_id);
                        editor.putString(Helper.CLIENT_SECRET, client_secret);
                        editor.apply();
                        Intent intent = new Intent(LoginActivity.this, WebviewConnectActivity.class);
                        Bundle b = new Bundle();
                        b.putString("url", "https://" + Helper.getPeertubeUrl(finalHost) + "/plugins/auth-openid-connect/0.0.1/auth/openid-connect");
                        intent.putExtras(b);
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        startActivity(intent);
                    } catch (Exception e) {
                        e.printStackTrace();
                        runOnUiThread(() -> {
                            binding.loginButton.setEnabled(true);
                            Toasty.error(LoginActivity.this, getString(R.string.toast_error), Toast.LENGTH_LONG).show();
                        });
                    }

                }).start();
            } else {
                new Thread(() -> {
                    Oauth oauth = new RetrofitPeertubeAPI(LoginActivity.this, finalInstance, null).oauthClient(Helper.CLIENT_NAME_VALUE, Helper.WEBSITE_VALUE, Helper.OAUTH_SCOPES_PEERTUBE, Helper.WEBSITE_VALUE);
                    if (oauth == null) {
                        runOnUiThread(() -> {
                            binding.loginButton.setEnabled(true);
                            Toasty.error(LoginActivity.this, getString(R.string.client_error), Toast.LENGTH_LONG).show();
                        });
                        return;
                    }

                    client_id = oauth.getClient_id();
                    client_secret = oauth.getClient_secret();

                    SharedPreferences sharedpreferences = getSharedPreferences(Helper.APP_PREFS, MODE_PRIVATE);
                    SharedPreferences.Editor editor = sharedpreferences.edit();
                    editor.putString(Helper.CLIENT_ID, client_id);
                    editor.putString(Helper.CLIENT_SECRET, client_secret);
                    editor.apply();
                    OauthParams oauthParams = new OauthParams();
                    oauthParams.setClient_id(client_id);
                    oauthParams.setClient_secret(client_secret);
                    oauthParams.setGrant_type("password");
                    oauthParams.setScope("user");
                    oauthParams.setUsername(binding.loginUid.getText().toString().trim());
                    if (binding.loginPasswd.getText() != null) {
                        oauthParams.setPassword(binding.loginPasswd.getText().toString());
                    }
                    try {
                        Token token = new RetrofitPeertubeAPI(LoginActivity.this, finalInstance, null).manageToken(oauthParams);
                        proceedLogin(token, finalHost);
                    } catch (final Exception | Error e) {
                        oauthParams.setUsername(binding.loginUid.getText().toString().toLowerCase().trim());
                        try {
                            Token token = new RetrofitPeertubeAPI(LoginActivity.this, finalInstance, null).manageToken(oauthParams);
                            proceedLogin(token, finalHost);
                        } catch (Error error) {
                            Error.displayError(LoginActivity.this, error);
                            error.printStackTrace();
                            runOnUiThread(() -> binding.loginButton.setEnabled(true));
                        }
                    }
                }).start();
            }

        });
    }

    @SuppressLint("ApplySharedPref")
    private void proceedLogin(Token token, String host) {
        runOnUiThread(() -> {
            if (token != null) {
                SharedPreferences sharedpreferences = getSharedPreferences(Helper.APP_PREFS, MODE_PRIVATE);
                SharedPreferences.Editor editor = sharedpreferences.edit();
                editor.putString(Helper.PREF_KEY_OAUTH_TOKEN, token.getAccess_token());
                editor.putString(Helper.PREF_INSTANCE, host);
                editor.commit();
                //Update the account with the token;
                updateCredential(LoginActivity.this, token.getAccess_token(), client_id, client_secret, token.getRefresh_token(), host);
            } else {
                binding.loginButton.setEnabled(true);
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }


}