package app.fedilab.fedilabtube.fragment;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import app.fedilab.fedilabtube.BuildConfig;
import app.fedilab.fedilabtube.MainActivity;
import app.fedilab.fedilabtube.R;
import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.RetrofitPeertubeAPI;
import app.fedilab.fedilabtube.client.data.ChannelData;
import app.fedilab.fedilabtube.client.data.VideoData;
import app.fedilab.fedilabtube.client.data.VideoPlaylistData;
import app.fedilab.fedilabtube.client.entities.PlaylistExist;
import app.fedilab.fedilabtube.drawer.AccountsHorizontalListAdapter;
import app.fedilab.fedilabtube.drawer.PeertubeAdapter;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.viewmodel.AccountsVM;
import app.fedilab.fedilabtube.viewmodel.PlaylistsVM;
import app.fedilab.fedilabtube.viewmodel.RelationshipVM;
import app.fedilab.fedilabtube.viewmodel.SearchVM;
import app.fedilab.fedilabtube.viewmodel.TimelineVM;
import es.dmoral.toasty.Toasty;

import static app.fedilab.fedilabtube.viewmodel.TimelineVM.TimelineType.VIDEOS_IN_LOCAL_PLAYLIST;


public class DisplayVideosFragment extends Fragment implements AccountsHorizontalListAdapter.EventListener, PeertubeAdapter.RelationShipListener, PeertubeAdapter.PlaylistListener {


    private LinearLayoutManager mLayoutManager;
    private GridLayoutManager gLayoutManager;
    private boolean flag_loading;
    private Context context;
    private PeertubeAdapter peertubeAdapater;
    private AccountsHorizontalListAdapter accountsHorizontalListAdapter;
    private String max_id, max_id_accounts;
    private List<VideoData.Video> peertubes;
    private List<ChannelData.Channel> channels;
    private TimelineVM.TimelineType type;
    private RelativeLayout mainLoader, nextElementLoader, textviewNoAction;
    private boolean firstLoad;
    private SwipeRefreshLayout swipeRefreshLayout;
    private SharedPreferences sharedpreferences;
    private String search_peertube;
    private TextView textviewNoActionText;
    private View rootView;
    private RecyclerView lv_status;
    private boolean check_ScrollingUp;
    private String forAccount;
    private ConstraintLayout top_account_container;
    private TimelineVM viewModelFeeds;
    private SearchVM viewModelSearch;
    private AccountsVM viewModelAccounts;
    private String channelId;
    private Map<String, Boolean> relationship;
    private Map<String, List<PlaylistExist>> playlists;
    private String playlistId;
    private String remoteInstance;
    private boolean sepiaSearch;
    private String startDate, endDate;

    public DisplayVideosFragment() {
    }


    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.fragment_video, container, false);


        peertubes = new ArrayList<>();
        channels = new ArrayList<>();
        context = getContext();
        startDate = null;
        endDate = null;
        Bundle bundle = this.getArguments();
        if (bundle != null) {
            search_peertube = bundle.getString("search_peertube", null);
            channelId = bundle.getString("channelId", null);
            remoteInstance = bundle.getString("peertube_instance", null);
            sepiaSearch = bundle.getBoolean("sepia_search", false);
            type = (TimelineVM.TimelineType) bundle.get(Helper.TIMELINE_TYPE);
            playlistId = bundle.getString("playlistId", null);
            startDate = bundle.getString("startDate", null);
            endDate = bundle.getString("endDate", null);
        }
        max_id = "0";
        forAccount = type == TimelineVM.TimelineType.ACCOUNT_VIDEOS ? channelId : null;
        lv_status = rootView.findViewById(R.id.lv_status);
        RecyclerView lv_accounts = rootView.findViewById(R.id.lv_accounts);
        Button display_all = rootView.findViewById(R.id.display_all);
        top_account_container = rootView.findViewById(R.id.top_account_container);
        max_id_accounts = null;
        flag_loading = true;
        firstLoad = true;
        check_ScrollingUp = false;

        assert context != null;
        sharedpreferences = context.getSharedPreferences(Helper.APP_PREFS, Context.MODE_PRIVATE);
        swipeRefreshLayout = rootView.findViewById(R.id.swipeContainer);

        mainLoader = rootView.findViewById(R.id.loader);
        nextElementLoader = rootView.findViewById(R.id.loading_next_status);
        textviewNoAction = rootView.findViewById(R.id.no_action);
        textviewNoActionText = rootView.findViewById(R.id.no_action_text);
        mainLoader.setVisibility(View.VISIBLE);
        nextElementLoader.setVisibility(View.GONE);

        peertubeAdapater = new PeertubeAdapter(this.peertubes, type, sepiaSearch);
        peertubeAdapater.playlistListener = this;
        peertubeAdapater.relationShipListener = this;
        lv_status.setAdapter(peertubeAdapater);

        accountsHorizontalListAdapter = new AccountsHorizontalListAdapter(this.channels, this);
        LinearLayoutManager layoutManager
                = new LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false);
        lv_accounts.setLayoutManager(layoutManager);
        lv_accounts.setAdapter(accountsHorizontalListAdapter);
        if (!Helper.isTablet(context)) {
            mLayoutManager = new LinearLayoutManager(context);
            lv_status.setLayoutManager(mLayoutManager);
        } else {
            gLayoutManager = new GridLayoutManager(context, 2);
            int spanCount = (int) Helper.convertDpToPixel(2, context);
            int spacing = (int) Helper.convertDpToPixel(5, context);
            lv_status.addItemDecoration(new GridSpacingItemDecoration(spanCount, spacing, true));
            lv_status.setLayoutManager(gLayoutManager);
        }
        viewModelAccounts = new ViewModelProvider(DisplayVideosFragment.this).get(AccountsVM.class);
        viewModelFeeds = new ViewModelProvider(DisplayVideosFragment.this).get(TimelineVM.class);
        viewModelSearch = new ViewModelProvider(DisplayVideosFragment.this).get(SearchVM.class);
        swipeRefreshLayout.setOnRefreshListener(() -> pullToRefresh(true));

        lv_accounts.addOnScrollListener(new RecyclerView.OnScrollListener() {
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                int firstVisibleItem = layoutManager.findFirstVisibleItemPosition();
                if (dy > 0) {
                    int visibleItemCount = layoutManager.getChildCount();
                    int totalItemCount = layoutManager.getItemCount();
                    if (firstVisibleItem + visibleItemCount == totalItemCount && context != null) {
                        viewModelAccounts.getAccounts(RetrofitPeertubeAPI.DataType.SUBSCRIBER, max_id_accounts).observe(DisplayVideosFragment.this.requireActivity(), apiResponse -> manageViewAccounts(apiResponse));
                    }
                }
            }
        });


        if (type != VIDEOS_IN_LOCAL_PLAYLIST) {
            lv_status.addOnScrollListener(new RecyclerView.OnScrollListener() {
                public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                    if (type == TimelineVM.TimelineType.SUBSCRIBTIONS) {
                        if (dy > 0) {
                            if (check_ScrollingUp) {
                                top_account_container.setVisibility(View.GONE);
                                final Handler handler = new Handler();
                                handler.postDelayed(() -> check_ScrollingUp = false, 300);

                            }
                        } else {
                            if (!check_ScrollingUp) {
                                top_account_container.setVisibility(View.VISIBLE);
                                final Handler handler = new Handler();
                                handler.postDelayed(() -> check_ScrollingUp = true, 300);
                            }
                        }
                    }
                    if (mLayoutManager != null) {
                        int firstVisibleItem = mLayoutManager.findFirstVisibleItemPosition();
                        if (dy > 0) {
                            int visibleItemCount = mLayoutManager.getChildCount();
                            int totalItemCount = mLayoutManager.getItemCount();
                            if (firstVisibleItem + visibleItemCount == totalItemCount && context != null) {
                                if (!flag_loading) {
                                    flag_loading = true;
                                    loadTimeline(max_id);
                                    nextElementLoader.setVisibility(View.VISIBLE);
                                }
                            } else {
                                nextElementLoader.setVisibility(View.GONE);
                            }
                        }
                    } else if (gLayoutManager != null) {
                        int firstVisibleItem = gLayoutManager.findFirstVisibleItemPosition();
                        if (dy > 0) {
                            int visibleItemCount = gLayoutManager.getChildCount();
                            int totalItemCount = gLayoutManager.getItemCount();
                            if (firstVisibleItem + visibleItemCount == totalItemCount && context != null) {
                                if (!flag_loading) {
                                    flag_loading = true;
                                    loadTimeline(max_id);
                                    nextElementLoader.setVisibility(View.VISIBLE);
                                }
                            } else {
                                nextElementLoader.setVisibility(View.GONE);
                            }
                        }
                    }
                }
            });
        }
        if (type == TimelineVM.TimelineType.SUBSCRIBTIONS) {
            AccountsVM viewModel = new ViewModelProvider(this).get(AccountsVM.class);
            viewModel.getAccounts(RetrofitPeertubeAPI.DataType.SUBSCRIBER, max_id).observe(DisplayVideosFragment.this.requireActivity(), this::manageViewAccounts);
        }
        loadTimeline(max_id);
        display_all.setOnClickListener(v -> {
            forAccount = null;
            pullToRefresh(false);
        });
        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        swipeRefreshLayout.setEnabled(true);
    }


    @Override
    public void onPause() {
        super.onPause();
        if (swipeRefreshLayout != null) {
            swipeRefreshLayout.setEnabled(false);
            swipeRefreshLayout.setRefreshing(false);
            swipeRefreshLayout.clearAnimation();
        }
        if (getActivity() != null) {
            InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
            if (imm != null && getView() != null) {
                imm.hideSoftInputFromWindow(getView().getWindowToken(), 0);
            }
        }
    }


    @Override
    public void onCreate(Bundle saveInstance) {
        super.onCreate(saveInstance);
    }


    @Override
    public void onAttach(@NotNull Context context) {
        super.onAttach(context);
        this.context = context;
    }


    @Override
    public void onStop() {
        super.onStop();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }


    private void manageViewAccounts(APIResponse apiResponse) {
        if (apiResponse != null && apiResponse.getChannels() != null && apiResponse.getChannels().size() > 0) {
            if (top_account_container.getVisibility() == View.GONE) {
                top_account_container.setVisibility(View.VISIBLE);
            }
            int previousPosition = channels.size();
            channels.addAll(apiResponse.getChannels());
            accountsHorizontalListAdapter.notifyItemRangeInserted(previousPosition, apiResponse.getChannels().size());
            if (max_id_accounts == null) {
                max_id_accounts = "0";
            }
            //max_id_accounts needs to work like an offset
            int tootPerPage = sharedpreferences.getInt(Helper.SET_VIDEOS_PER_PAGE, Helper.VIDEOS_PER_PAGE);
            max_id_accounts = String.valueOf(Integer.parseInt(max_id_accounts) + tootPerPage);
        }
    }


    private void manageVIewVideos(APIResponse apiResponse) {
        //hide loaders
        mainLoader.setVisibility(View.GONE);
        nextElementLoader.setVisibility(View.GONE);
        //handle other API error
        if (this.peertubes == null || apiResponse == null || (apiResponse.getError() != null)) {
            if (apiResponse == null)
                Toasty.error(context, context.getString(R.string.toast_error), Toast.LENGTH_LONG).show();
            else {
                Toasty.error(context, apiResponse.getError().getError(), Toast.LENGTH_LONG).show();
            }
            swipeRefreshLayout.setRefreshing(false);
            flag_loading = false;
            return;
        }
        int previousPosition = this.peertubes.size();
        if (max_id == null)
            max_id = "0";
        //max_id needs to work like an offset
        int videoPerPage = sharedpreferences.getInt(Helper.SET_VIDEOS_PER_PAGE, Helper.VIDEOS_PER_PAGE);
        max_id = String.valueOf(Integer.parseInt(max_id) + videoPerPage);
        if (apiResponse.getPeertubes() == null && apiResponse.getVideoPlaylist() == null) {
            return;
        }
        if (apiResponse.getVideoPlaylist() != null) {
            apiResponse.setPeertubes(new ArrayList<>());
            for (VideoPlaylistData.VideoPlaylist v : apiResponse.getVideoPlaylist()) {
                apiResponse.getPeertubes().add(v.getVideo());
            }
        }
        if (!BuildConfig.google_restriction) {
            this.peertubes.addAll(apiResponse.getPeertubes());
        } else {
            for (VideoData.Video video : apiResponse.getPeertubes()) {
                if (video.getName() == null || !video.getName().toLowerCase().contains("youtube") || !video.getName().toLowerCase().contains("download")) {
                    this.peertubes.add(video);
                }
            }
        }


        //If no item were inserted previously the adapter is created
        if (previousPosition == 0) {
            peertubeAdapater = new PeertubeAdapter(this.peertubes, type, sepiaSearch);
            peertubeAdapater.playlistListener = DisplayVideosFragment.this;
            peertubeAdapater.relationShipListener = DisplayVideosFragment.this;
            lv_status.setAdapter(peertubeAdapater);
        } else
            peertubeAdapater.notifyItemRangeInserted(previousPosition, apiResponse.getPeertubes().size());
        //remove handlers
        swipeRefreshLayout.setRefreshing(false);
        textviewNoAction.setVisibility(View.GONE);
        if (firstLoad && (apiResponse.getPeertubes() == null || apiResponse.getPeertubes().size() == 0)) {
            textviewNoActionText.setText(R.string.no_video_to_display);
            textviewNoAction.setVisibility(View.VISIBLE);
        }
        flag_loading = false;
        firstLoad = false;

        if (Helper.isLoggedIn(context)) {
            List<String> uids = new ArrayList<>();
            for (VideoData.Video video : apiResponse.getPeertubes()) {
                if (video != null) {
                    uids.add(video.getChannel().getName() + "@" + video.getChannel().getHost());
                }
            }
            if (uids.size() > 0 && !DisplayVideosFragment.this.isDetached()) {
                try {
                    RelationshipVM viewModel = new ViewModelProvider(this).get(RelationshipVM.class);
                    viewModel.get(uids).observe(DisplayVideosFragment.this.requireActivity(), this::manageVIewRelationship);
                } catch (Exception ignored) {
                }
            }

            List<String> videoIds = new ArrayList<>();
            for (VideoData.Video video : apiResponse.getPeertubes()) {
                if (video != null) {
                    videoIds.add(video.getId());
                }
            }
            if (videoIds.size() > 0 && !DisplayVideosFragment.this.isDetached()) {
                try {
                    PlaylistsVM viewModel = new ViewModelProvider(this).get(PlaylistsVM.class);
                    viewModel.videoExists(videoIds).observe(DisplayVideosFragment.this.requireActivity(), this::manageVIewPlaylist);
                } catch (Exception ignored) {
                }
            }

        }
    }

    public void manageVIewPlaylist(APIResponse apiResponse) {
        if (apiResponse.getError() != null || apiResponse.getVideoExistPlaylist() == null) {
            return;
        }
        if (playlists == null) {
            playlists = new HashMap<>();
        }
        playlists.putAll(apiResponse.getVideoExistPlaylist());
        for (VideoData.Video video : peertubes) {
            if (video != null) {
                video.setPlaylistExists(playlists.get(video.getId()));
            }
        }

    }

    public void manageVIewRelationship(APIResponse apiResponse) {
        if (apiResponse.getError() != null) {
            return;
        }
        if (relationship == null) {
            relationship = new HashMap<>();
        }
        relationship.putAll(apiResponse.getRelationships());
    }

    @Override
    public void onDestroyView() {
        if (lv_status != null) {
            try {
                lv_status.setAdapter(null);
            } catch (Exception ignored) {
            }
        }
        super.onDestroyView();
        rootView = null;
    }


    public void scrollToTop() {
        if (mLayoutManager != null) {
            mLayoutManager.scrollToPositionWithOffset(0, 0);
        } else if (gLayoutManager != null) {
            gLayoutManager.scrollToPositionWithOffset(0, 0);
        }
    }


    public void pullToRefresh(boolean reload) {
        if (type == TimelineVM.TimelineType.SUBSCRIBTIONS && reload) {
            Fragment fragment = ((AppCompatActivity) context).getSupportFragmentManager().findFragmentByTag("2");
            if (fragment != null) {
                if (context instanceof MainActivity) {
                    FragmentManager fm = ((MainActivity) context).getSupportFragmentManager();
                    fm.beginTransaction().remove(fragment).commit();
                    DisplayVideosFragment subscriptionFragment = new DisplayVideosFragment();
                    Bundle bundle = new Bundle();
                    bundle.putSerializable(Helper.TIMELINE_TYPE, TimelineVM.TimelineType.SUBSCRIBTIONS);
                    subscriptionFragment.setArguments(bundle);
                    ((MainActivity) context).setActive(subscriptionFragment);
                    ((MainActivity) context).setSubscriptionFragment(subscriptionFragment);
                    fm.beginTransaction().add(R.id.nav_host_fragment, subscriptionFragment, "2").commit();
                }
            }

        } else {
            int size = peertubes.size();
            peertubes.clear();
            peertubes = new ArrayList<>();
            max_id = "0";
            peertubeAdapater.notifyItemRangeRemoved(0, size);
            if (forAccount == null) {
                for (ChannelData.Channel channel : channels) {
                    channel.setSelected(false);
                }
                accountsHorizontalListAdapter.notifyItemRangeRemoved(0, channels.size());
            }
            loadTimeline("0");
        }

    }

    @Override
    public void click(String forAccount) {
        this.forAccount = forAccount;
        pullToRefresh(false);
    }

    /**
     * Manage timeline load
     *
     * @param max_id String pagination
     */
    private void loadTimeline(String max_id) {
        if (search_peertube == null) { //Not a Peertube search
            if (type == TimelineVM.TimelineType.CHANNEL_VIDEOS) {
                viewModelFeeds.getVideosInChannel(sepiaSearch ? remoteInstance : null, channelId, max_id).observe(this.requireActivity(), this::manageVIewVideos);
            } else if (type == TimelineVM.TimelineType.VIDEOS_IN_PLAYLIST) {
                viewModelFeeds.loadVideosInPlaylist(playlistId, max_id).observe(this.requireActivity(), this::manageVIewVideos);
            } else if (type == VIDEOS_IN_LOCAL_PLAYLIST) {
                viewModelFeeds.loadVideosInLocalPlaylist(playlistId).observe(this.requireActivity(), this::manageVIewVideos);
            } else if (type == TimelineVM.TimelineType.HISTORY) {
                viewModelFeeds.getVideoHistory(max_id, startDate, endDate).observe(this.requireActivity(), this::manageVIewVideos);
            } else {
                viewModelFeeds.getVideos(type, max_id, forAccount).observe(this.requireActivity(), this::manageVIewVideos);
            }
        } else {
            viewModelSearch.getVideos(max_id, search_peertube).observe(this.requireActivity(), this::manageVIewVideos);
        }
    }


    @Override
    public Map<String, Boolean> getRelationShip() {
        return relationship;
    }

    @Override
    public Map<String, List<PlaylistExist>> getPlaylist() {
        return playlists;
    }


    static class GridSpacingItemDecoration extends RecyclerView.ItemDecoration {

        private final int spanCount;
        private final int spacing;
        private final boolean includeEdge;

        public GridSpacingItemDecoration(int spanCount, int spacing, boolean includeEdge) {
            this.spanCount = spanCount;
            this.spacing = spacing;
            this.includeEdge = includeEdge;
        }

        @Override
        public void getItemOffsets(@NotNull Rect outRect, @NotNull View view, RecyclerView parent, @NotNull RecyclerView.State state) {
            int position = parent.getChildAdapterPosition(view);
            int column = position % spanCount;

            if (includeEdge) {
                outRect.left = spacing - column * spacing / spanCount;
                outRect.right = (column + 1) * spacing / spanCount;

                if (position < spanCount) {
                    outRect.top = spacing;
                }
                outRect.bottom = spacing;
            } else {
                outRect.left = column * spacing / spanCount;
                outRect.right = spacing - (column + 1) * spacing / spanCount;
                if (position >= spanCount) {
                    outRect.top = spacing;
                }
            }
        }
    }
}
